@@ -1,9 +1,11 @@
+import re
 from pprint import pprint
 import tensorflow as tf
 import tensorflow.keras as keras
 from tensorflow.keras import layers
 from tensorflow.keras import activations
 from . import load
+import tfbox.utils.helpers as h
 #
 # CONSTANTS
 #
@@ -16,18 +18,29 @@ DEFAULT_MAX_POOLING={
     'padding': "same"
 }
 DEFAULT_GLOBAL_POOLING='average'
-
-
-
+UPSAMPLE_ERROR=(
+    'upsample called with scale==1. '
+    'Use allow_identity=True to force.'
+)
 
 #
 # HELPERS
 #
-ACTIVATIONS={
-    'relu': layers.ReLU,
-    'sigmoid': activations.sigmoid,
-    'softmax': layers.Softmax
-}
+def get(key,activation=False):
+    if activation:
+        return ACTIVATIONS[h.snake(key)]
+    else:
+        return BLOCKS[h.snake(key)]
+
+
+def layer_name(*name_path,index=None,named=True):
+    if named and name_path:
+        name_path=[p for p in name_path if p] 
+        name='.'.join(name_path)
+        if index is not None:
+            name=f'{name}-{index}'
+        return name
+
 
 def get_activation(act,**config):
     if act:
@@ -47,35 +60,32 @@ def upsample(
         rescale=1,
         mode='bilinear',
         allow_resize=True,
-        force_resize=False):
+        force_resize=False,
+        allow_identity=False):
     if scale is None:
         if shape is None:
             shape=like.shape
         scale=rescale*shape[2]/x.shape[2]
-    if allow_resize and (force_resize or (scale!=int(scale))):
-        h=round(x.shape[1]*scale)
-        w=round(x.shape[2]*scale)
-        return tf.image.resize(x,(h,w),method=mode)
+    if scale==1:
+        if not allow_identity:
+            raise ValueError(UPSAMPLE_ERROR)
     else:
-        scale=int(scale)  
-        return layers.UpSampling2D(
-            size=(scale,scale),
-            interpolation=mode)(x)
-
-
-def layer_name(*name_path,index=None,named=True):
-    if named and name_path:
-        name_path=[p for p in name_path if p] 
-        name='.'.join(name_path)
-        if index is not None:
-            name=f'{name}-{index}'
-        return name
+        if allow_resize and (force_resize or (scale!=int(scale))):
+            h=round(x.shape[1]*scale)
+            w=round(x.shape[2]*scale)
+            x=tf.image.resize(x,(h,w),method=mode)
+        else:
+            scale=int(scale)
+            x=layers.UpSampling2D(
+                size=(scale,scale),
+                interpolation=mode)(x)
+    return x
 
 
 #
 # GENERAL BLOCKS 
 #
-class CBAD(keras.Model):
+class Conv(keras.Model):
     """ Conv-BatchNorm-Activation-Dropout
     """
     #
@@ -94,11 +104,11 @@ class CBAD(keras.Model):
             dropout=DEFAULT_DROPOUT,
             dropout_config={},
             act_last=False,
-            keep_output=False,
+            is_skip=False,
             name=None,
             named_layers=True,
             **conv_config):
-        super(CBAD, self).__init__()
+        super(Conv, self).__init__()
         self.block_name=name or self.name
         self.named_layers=named_layers
         if seperable:
@@ -123,7 +133,7 @@ class CBAD(keras.Model):
         self.act=self._activation(act,act_config)
         self.do=self._dropout(dropout,dropout_config)
         self.act_last=act_last
-        self.keep_output=keep_output
+        self.is_skip=is_skip
 
 
     def __call__(self,x,training=False):
@@ -142,7 +152,7 @@ class CBAD(keras.Model):
     # INTERNAL
     #
     def _layer_name(self,name):
-        return layer_name(*[self.block_name,name],named=self.named_layers)
+        return layer_name(self.block_name,name,named=self.named_layers)
 
 
     def _batch_norm(self,batch_norm):
@@ -166,8 +176,8 @@ class CBAD(keras.Model):
 
 
 
-class CBADStack(keras.Model):
-    """ (Res)Stack of CBAD Blocks
+class Stack(keras.Model):
+    """ (Res)Stack of Conv Blocks
     """
     #
     # CONSTANTS
@@ -200,12 +210,12 @@ class CBADStack(keras.Model):
             dilation_rate=1,
             output_stride=1,
             max_pooling=False,
-            keep_output=False,
+            is_skip=False,
             name=None,
             named_layers=True,
-            layers_name='cbad',
+            layers_name='conv',
             **conv_config):
-        super(CBADStack, self).__init__()
+        super(Stack, self).__init__()
         self.block_name=name or self.name
         self.named_layers=named_layers
         self.layers_name=layers_name
@@ -246,11 +256,11 @@ class CBADStack(keras.Model):
             self.filters_list[-1],
             output_stride)
         self.stack=self._build_stack(output_stride)
-        self.keep_output=keep_output
+        self.is_skip=is_skip
 
 
     def __call__(self,x,training=False,**kwargs):
-        if self.residual==CBADStack.IDENTITY:
+        if self.residual==Stack.IDENTITY:
             res=x
         elif self.residual:
             res=self.residual(x)
@@ -321,16 +331,16 @@ class CBADStack(keras.Model):
 
 
     def _layer_name(self,name,index=None):
-        return layer_name(*[self.block_name,name],index=index,named=self.named_layers)
+        return layer_name(self.block_name,name,index=index,named=self.named_layers)
 
 
     def _residual(self,residual,residual_act,filters,output_stride):
-        if residual and (residual!=CBADStack.IDENTITY):
+        if residual and (residual!=Stack.IDENTITY):
             if not residual_act:
                 act=False
             else:
                 act=self.act
-            residual=CBAD(
+            residual=Conv(
                 filters=filters,
                 kernel_size=1,
                 strides=output_stride,
@@ -350,7 +360,7 @@ class CBADStack(keras.Model):
                 strides=output_stride
             else:
                 strides=1
-            _layers.append(CBAD(
+            _layers.append(Conv(
                 filters=f,
                 kernel_size=k,
                 strides=strides,
@@ -369,8 +379,8 @@ class CBADStack(keras.Model):
 
 
 
-class CBADGroup(keras.Model):
-    """ (Res)Parallel Group of CBAD Blocks
+class Group(keras.Model):
+    """ (Res)Parallel Group of Conv Blocks
     """
     #
     # CONSTANTS
@@ -398,12 +408,12 @@ class CBADGroup(keras.Model):
             act_last=False,
             padding='same',
             out_config={},
-            keep_output=False,
+            is_skip=False,
             name=None,
             named_layers=True,
-            layers_name='cbad',
+            layers_name='conv',
             **conv_config):
-        super(CBADGroup, self).__init__()
+        super(Group, self).__init__()
         if padding is not 'same':
             raise NotImplementedError('currently only accepts padding=same')
         self.block_name=name or self.name
@@ -422,11 +432,11 @@ class CBADGroup(keras.Model):
         self.pooling_stack=self._global_pooling(global_pooling)
         self.group=self._build_group()
         self.out_conv=self._out_conv(kernel_size_out,out_config)
-        self.keep_output=keep_output
+        self.is_skip=is_skip
 
 
     def __call__(self,x,training=False,**kwargs):
-        if self.residual==CBADGroup.IDENTITY:
+        if self.residual==Group.IDENTITY:
             res=x
         elif self.residual:
             res=self.residual(x)
@@ -445,7 +455,7 @@ class CBADGroup(keras.Model):
     # INTERNAL
     #
     def _layer_name(self,name,index=None):
-        return layer_name(*[self.block_name,name],index=index,named=self.named_layers)
+        return layer_name(self.block_name,name,index=index,named=self.named_layers)
 
 
     def _call_group(self,x):
@@ -505,12 +515,12 @@ class CBADGroup(keras.Model):
 
 
     def _residual(self,residual,residual_act):
-        if residual and (residual!=CBADStack.IDENTITY):
+        if residual and (residual!=Stack.IDENTITY):
             if not residual_act:
                 act=False
             else:
                 act=self.act
-            residual=CBAD(
+            residual=Conv(
                 filters=self.hidden_filters,
                 kernel_size=1,
                 act=act,
@@ -524,7 +534,7 @@ class CBADGroup(keras.Model):
     def _build_group(self):
         _layers=[] 
         for i,(k,d) in enumerate(zip(self.kernel_size_list,self.dilation_rate_list)):
-            _layers.append(CBAD(
+            _layers.append(Conv(
                 filters=self.hidden_filters,
                 kernel_size=k,
                 dilation_rate=d,
@@ -542,7 +552,7 @@ class CBADGroup(keras.Model):
             config['act']=self.act
             config['act_config']=self.act_config
             config.update(out_config)
-            return CBAD(
+            return Conv(
                 filters=self.filters,
                 kernel_size=kernel_size,
                 name=self._layer_name('out_conv'),
@@ -619,7 +629,7 @@ class Residual(keras.Model):
             if identity:
                 residual=Residual.IDENTITY
             else:
-                residual=CBAD(
+                residual=Conv(
                     filters=filters,
                     kernel_size=1,
                     **config)
@@ -630,8 +640,8 @@ class Residual(keras.Model):
 #
 # PRE-CONFIGURED BLOCKS
 #
-class ASPP(CBADGroup):
-    """ ASPP (convenience wrapper for CBADGroup)
+class ASPP(Group):
+    """ ASPP (convenience wrapper for Group)
     """
     #
     # CONSTANTS
@@ -718,7 +728,7 @@ class SegmentClassifier(keras.Model):
             nb_classes,
             len(kernel_size_list))
         if len(filters_list)>1:
-            self.preclassifier=CBADStack(
+            self.preclassifier=Stack(
                 filters_list=filters_list[:-1],
                 kernel_size_list=kernel_size_list[:-1],
                 seperable=seperable_preclassification,
@@ -730,7 +740,7 @@ class SegmentClassifier(keras.Model):
             self.preclassifier=False
         act=self._activation(nb_classes,output_act)
         print('OUTPUT_ACTIVATION:',act,output_act)
-        self.classifier=CBAD(
+        self.classifier=Conv(
                 filters=filters_list[-1],
                 kernel_size=kernel_size_list[-1],
                 batch_norm=output_batch_norm,
@@ -752,7 +762,7 @@ class SegmentClassifier(keras.Model):
     # INTERNAL
     #
     def _layer_name(self,name,index=None):
-        return layer_name(*[self.block_name,name],index=index,named=self.named_layers)
+        return layer_name(self.block_name,name,index=index,named=self.named_layers)
 
 
     def _filters_list(self,filters_list,filters,nb_classes,depth):
@@ -781,3 +791,23 @@ class SegmentClassifier(keras.Model):
         return act
 
 
+#
+# TBOX DICTS
+#
+BLOCKS={
+    'conv': Conv,
+    'stack': Stack,
+    'group': Group,
+    'squeeze_excitation': SqueezeExcitation,
+    'aspp': ASPP,
+    'segment_classifier': SegmentClassifier
+}
+ACTIVATIONS={
+    'relu': layers.ReLU,
+    'sigmoid': activations.sigmoid,
+    'softmax': layers.Softmax
+}
+
+
+
+
