@@ -1,4 +1,5 @@
 import os
+import re
 import math
 import yaml
 import tensorflow.keras as keras
@@ -31,6 +32,13 @@ def read_json(path,*key_path):
     return obj
 
 
+#
+# PYTHON
+#
+_SNAKE_RGX=re.compile('((?<=[a-z0-9])[A-Z]|(?!^)[A-Z](?=[a-z]))')
+def snake(string):
+    return _SNAKE_RGX.sub(r'_\1',str(string)).lower()
+
 
 #
 # UTILS
@@ -176,6 +184,40 @@ def set_trainable(
             return matched
 
 
+#
+# CONSTANTS
+#
+COLORS=[
+    '#8dd3c7',
+    '#ffffb3',
+    '#bebada',
+    '#fb8072',
+    '#80b1d3',
+    '#fdb462',
+    '#b3de69',
+    '#fccde5',
+    '#d9d9d9',
+    '#bc80bd',
+    '#ccebc5',
+    '#ffed6f',
+    '#e41a1c',
+    '#377eb8',
+    '#4daf4a',
+    '#984ea3',
+    '#ff7f00',
+    '#ffff33',
+    '#a65628',
+    '#f781bf',
+    '#999999',
+    '#66c2a5',
+    '#fc8d62',
+    '#8da0cb',
+    '#e78ac3',
+    '#a6d854',
+    '#ffd92f',
+    '#e5c494',
+    '#b3b3b3'
+]
 
 #
 # INTERNAL
